
// bibhtml: Generates HTML page (with images) from BibTex files
// See: https://www.telecom-paris.fr/~elc/software/
//
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef __BibTex__
#define __BibTex__

#include <string>
#include <fstream>


/** BibTex Writer: Writes an entry or a list of entries to a BibTex file.
 */
class BibWriter {
public:  
  /// Writes a list of entries to a BibTex file.
  bool write(const std::string& filename, const BibEntries&);
  
  ///  Writes an entry to a BibTex file.
  void write(std::ostream&, const BibEntry&, bool inHtmlFile);
};


/** BibTex Reader: Reads the entries of a BibTex file.
 */
class BibReader {
public:  
  /** reads the entries of a BibTex file.
   *  Creates the corresponding Entry objects and adds them to the entry database.
   */
  BibReader(const std::string& filename, bool isAuxiliaryFile = false);
  
  /// returns true if no errors.
  bool status() {return status_;}

protected:
  bool readEntry();
  char readField();
  char readSlotName(std::string&, char firstChar);
  char readUnquotedValue(std::string&, char firstChar);
  char readQuotedValue(std::string&);
  char readBracedValue(std::string&);
  char nextChar();
  char nextChar(std::string& value);
  void error(const std::string& msg);
  void parseAuthors(const std::string& value, std::string& parsed);
  static void parseAuthors(const std::string& authors_def, BibAuthors& authors);
  static void parseValue(const std::string& value, std::string& parsed);

private:
  std::string filename_;
  std::ifstream fin_;
  bool status_{false};
  class BibEntry* entry_{nullptr};
  bool isAuxiliary_{false};
  unsigned long lineNo_;
};

#endif
